"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var asynciterator_1 = require("asynciterator");
/**
 * An iterator that takes elements from a given set of iterators in a round-robin manner.
 *
 * Based on LDF client's UnionIterator:
 * https://github.com/LinkedDataFragments/Client.js/blob/master/lib/sparql/UnionIterator.js
 */
var RoundRobinUnionIterator = /** @class */ (function (_super) {
    tslib_1.__extends(RoundRobinUnionIterator, _super);
    function RoundRobinUnionIterator(sources, options) {
        var _this = _super.call(this, options || { autoStart: false }) || this;
        _this.currentSource = 0;
        _this.attachedListeners = false;
        _this.sources = sources;
        for (var _i = 0, _a = _this.sources; _i < _a.length; _i++) {
            var source = _a[_i];
            source.on('error', function (error) { return _this.emit('error', error); });
        }
        return _this;
    }
    RoundRobinUnionIterator.prototype._read = function (count, done) {
        var _this = this;
        if (!this.attachedListeners) {
            this.attachedListeners = true;
            for (var _i = 0, _a = this.sources; _i < _a.length; _i++) {
                var source = _a[_i];
                source.on('readable', function () { return _this._fillBuffer(); });
                source.on('end', function () { return _this._fillBuffer(); });
            }
        }
        var item = null;
        var attempts = this.sources.length;
        // Iterate over all sources once
        while (this.sources.length && item === null && attempts--) {
            var source = this.sources[this.currentSource];
            // Read from the current source
            item = source.read();
            // Remove the source if it has ended, otherwise, increment our stored position
            if (source.ended) {
                this.sources.splice(this.currentSource, 1);
            }
            else {
                this.currentSource++;
            }
            this.currentSource = this.currentSource % this.sources.length;
        }
        // Push to the buffer if we have an item
        if (item !== null) {
            this._push(item);
        }
        // Otherwise close
        if (!this.sources.length) {
            this.close();
        }
        done();
    };
    return RoundRobinUnionIterator;
}(asynciterator_1.BufferedIterator));
exports.RoundRobinUnionIterator = RoundRobinUnionIterator;
//# sourceMappingURL=RoundRobinUnionIterator.js.map